/*
  
  Emic 2 Text-to-Speech Module: Serial I/O Demo
  terry@yourduino.com  
  05/21/2013  V1.01

  
*/

// include the SoftwareSerial library so we can use it to talk to the Emic 2 module
#include <SoftwareSerial.h>

#define rxPin 3    // Serial input (connects to Emic 2 SOUT)
#define txPin 5    // Serial output (connects to Emic 2 SIN)
#define ledPin 13  // Most Arduino boards have an on-board LED on this pin

// set up a new serial port
SoftwareSerial emicSerial =  SoftwareSerial(rxPin, txPin);


void setup()  // Set up code called once on start-up
{
  // define pin modes
  pinMode(ledPin, OUTPUT);
  pinMode(rxPin, INPUT);
  pinMode(txPin, OUTPUT);
  
  Serial.begin(9600);  
  
  // set the data rate for the SoftwareSerial port
  emicSerial.begin(9600);

  digitalWrite(ledPin, LOW);  // turn LED off
  
  /*
    When the Emic 2 powers on, it takes about 3 seconds for it to successfully
    intialize. It then sends a ":" character to indicate it's ready to accept
    commands. If the Emic 2 is already initialized, a CR will also cause it
    to send a ":"
  */
  emicSerial.print('\n');             // Send a CR in case the system is already up
  while (emicSerial.read() != ':');   // When the Emic 2 has initialized and is ready, it will send a single ':' character, so wait here until we receive it
  delay(10);                          // Short delay
  emicSerial.flush();                 // Flush the receive buffer
  emicSerial.print("S System Started. This is Oh Lee.\n");  // Send a string to convert to speech
}

void loop()
{
  // Copy Serial Data from Serial Monitor to TTS
  while (Serial.available() > 0) 
  {
    emicSerial.write(Serial.read());
  }
  // Copy Serial Data from the TTS Module to the Serial Monitor
  while (emicSerial.available() > 0) 
  {
    Serial.write(emicSerial.read());
  }  
  
}// End LOOP
