/* Test Morse to Text-To-Speech
 
 - CONNECTIONS:
   - MorseIN 8  ( Not conflict with Software Serial)
   - TTS IN 5 TTS Out 3
 - V1.12 05/22/13
   Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/
#include <SoftwareSerial.h>  // For communication to Emic2 Text-To-Speech module
// Module from http://www.grandideastudio.com/portfolio/emic-2-text-to-speech-module/ 
// (Sold by Parallax and Sparkfun.com)

#include <avr/pgmspace.h>    // For Morse Decoder
// This library from:   http://raronoff.wordpress.com/2010/12/16/morse-endecoder/
#include <MorseEnDecoder.h>

/*-----( Declare Constants and Pin Numbers )-----*/
const byte morseInPin = 8;  // Needs to be in this form

#define rxPin 3    // Serial input (connects to Emic 2 SOUT)
#define txPin 5    // Serial output (connects to Emic 2 SIN)

/*-----( Declare objects )-----*/
morseDecoder morseInput(morseInPin, MORSE_KEYER, MORSE_ACTIVE_LOW);
SoftwareSerial emicSerial =  SoftwareSerial(rxPin, txPin);

/*-----( Declare Variables )-----*/
//none

void setup()   /****** SETUP: RUNS ONCE ******/
{
  pinMode(rxPin, INPUT);
  pinMode(txPin, OUTPUT);
  
  Serial.begin(9600); 
  delay(10);
  Serial.println("Morse to Text-To-Speech Demo");  
  //-----( SET UP MORSE DECODE )--------
  morseInput.setspeed(5);  // 5 words per minute : slow  
  
  //-----( SET UP Text-To-Speech )-----------
  emicSerial.begin(9600);             // set the data rate for the SoftwareSerial port
  emicSerial.print('\n');             // Send a CR in case the system is already up
  while (emicSerial.read() != ':');   // When the Emic 2 has initialized and is ready, it will send a single ':' character, so wait here until we receive it
  delay(10);                          // Short delay
  emicSerial.flush();                 // Flush the receive buffer
  emicSerial.print("S System Started. This is Oh Lee.\n");  // Send a string to convert to speech  
  
}//--(end setup )---

void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  //-------------------------( Decode Morse )---------------------------------
    // Need to call this once per loop
  morseInput.decode();
//  delay(25);   // Simulate real program loop...

//----( RECEIVE MORSE (INPUT) )-------------
  // If a character is decoded from the input, write it to serial port and TextToSpeech
  if (morseInput.available())
  {
    char receivedMorse = morseInput.read();
    Serial.print(receivedMorse);
    delay (10);
    emicSerial.write(receivedMorse);   // Write to TTS
    if (receivedMorse == 0x2E) // "."
      {
        emicSerial.write(0x0A); // End with LineFeed character to trigger TTS module
      }
    // A little error checking    
    if (receivedMorse == '#') Serial.println("< ERROR:too many morse signals! >");
  }
   
  
//------------------( Do typed-in Text-To-Speech )------------------------------

 // Copy Serial Data from Serial Monitor to TTS if some is typing it in
  while (Serial.available() > 0) 
  {
    emicSerial.write(Serial.read());
  } 

}//--(end main loop )---

/*-----( Declare User-written Functions )-----*/

//none
//*********( THE END )***********

