/* YourDuinoStarter Example: 3 State Machines blink
 - WHAT IT DOES
  - Runs 3 synchronized State Machines that blink at
    different times
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
   - Pin 13 LED
 - V1.00 12/06/2013
   Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/
#include <SM.h>
/*-----( Declare Constants and Pin Numbers )-----*/
#define PIN13_LED 13 // The onboard LED
#define BlinkMS 25   // How long to blink

/*-----( Declare objects )-----*/
SM SyncMachine(SyncStart);
SM FirstBlink(FB_done);
SM SecondBlink(SB_done);
SM ThirdBlink(TB_done);

/*-----( Declare Variables )-----*/
int FB_counter;
int SB_counter;
int TB_counter;

void setup()   /****** SETUP: RUNS ONCE ******/
{
  Serial.begin(115200);
  pinMode(PIN13_LED, OUTPUT);

}//--(end setup )---


void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  EXEC(SyncMachine);
  EXEC(FirstBlink);
  EXEC(SecondBlink);
  EXEC(ThirdBlink);  
}//--(end main loop )---

/*-----( Declare User-written Functions )-----*/

//---( STATE MACHINE: SyncMachine )------------------
State SyncStart()
{
  Serial.println("+++ Sync Start");
  SyncMachine.Set(SyncMachines) ;
}
State SyncMachines()
{
  Serial.println("SYNC START 3 Machines");
 
  FirstBlink.Set(FB_start);  // Start the three State Machines
  SecondBlink.Set(SB_start);
  ThirdBlink.Set(TB_start);  
  
  SyncMachine.Set(SyncWait);  // Go to the wait state
}

State SyncWait()
{
  if (SyncMachine.Timeout(2000)) SyncMachine.Set(SyncStart) ;
}

//-----------------------------------------



//---( STATE MACHINE: FirstBlink )------------------
State FB_start()    // Initial Delay from Sync
{
  Serial.println("FB Start");   
  FB_counter = 0;  
  if (FirstBlink.Timeout(250)) FirstBlink.Set(FB_on) ; 
}

State FB_on()
{
  digitalWrite(PIN13_LED, HIGH);  
  if (FirstBlink.Timeout(50)) FirstBlink.Set(FB_Count) ; 
}

State FB_Count()
{
  FB_counter ++;  // Count only once through the sequence!
  FirstBlink.Set(FB_off) ;  
}

State FB_off()
{
  digitalWrite(PIN13_LED, LOW);  
  if (FB_counter >= 1) FirstBlink.Set(FB_done) ;  
  if (FirstBlink.Timeout(50)) FirstBlink.Set(FB_on) ; 
}

State FB_done()
{
}

//---( STATE MACHINE: SecondBlink )------------------

State SB_start()    // Initial Delay from Sync
{
  Serial.println("SB Start"); 
  SB_counter = 0;  
  if (SecondBlink.Timeout(750)) SecondBlink.Set(SB_on) ; 
}

State SB_on()
{
  digitalWrite(PIN13_LED, HIGH);  
  if (SecondBlink.Timeout(50)) SecondBlink.Set(SB_Count) ; 
}

State SB_Count()
{
  SB_counter ++;
  SecondBlink.Set(SB_off) ;  
}

State SB_off()
{
  digitalWrite(PIN13_LED, LOW);  
  if (SB_counter >= 2) SecondBlink.Set(SB_done) ;  
  if (SecondBlink.Timeout(50)) SecondBlink.Set(SB_on) ; 
}

State SB_done()
{
}


//---( STATE MACHINE: ThirdBlink )------------------

State TB_start()    // Initial Delay from Sync
{
  Serial.println("TB Start"); 
  TB_counter = 0;    
  if (ThirdBlink.Timeout(1250)) ThirdBlink.Set(TB_on) ; 
}

State TB_on()
{
  digitalWrite(PIN13_LED, HIGH);  
  if (ThirdBlink.Timeout(50)) ThirdBlink.Set(TB_Count) ; 
}

State TB_Count()
{
  TB_counter ++;
  ThirdBlink.Set(TB_off) ;  
}

State TB_off()
{
  digitalWrite(PIN13_LED, LOW);  
  if (TB_counter >= 3) ThirdBlink.Set(TB_done) ;  
  if (ThirdBlink.Timeout(50)) ThirdBlink.Set(TB_on) ; 
}

State TB_done()
{
}

//*********( THE END )***********

