/* YourDuino.com Example Software Sketch
 YD_DS18B20_Temp_4LINE_LCD_New
 Displays Temperature from DS18B20 Sensor to New type 4x20 LCD
 NEW TYPE Marked "Arduino-IIC-LCD GY-LCD-V1"
 - SEE the comments after "//" on each line below
 - CONNECTIONS: (Holding chip with pins down, label towards you)
 - Left  pin:  Ground
 - Right pin:  +5V
 - Center pin: Arduino Pin 10 and ALSO:
 - Center pin: to 4700 (4.7K) or 5K resistor * (other end goes to +5V) 
 * NOTE: Two 10K resistors connected in parallel can be used. 
 SDA - ANALOG Pin 4
 SCL - ANALOG pin 5 
 Questions: terry@yourduino.com */
/*-----( Import needed libraries )-----*/ 
#include <OneWire.h>            // For many "1-wire" devices
#include <DallasTemperature.h>  // Specific for temperature sensors
#include <Wire.h>
#include <LCD.h>
#include <LiquidCrystal_I2C.h>  // F Malpartida's NewLiquidCrystal library
//Download: https://bitbucket.org/fmalpartida/new-liquidcrystal/downloads
// Move original LiquidCrystal library elsewhere, copy this in it's place

/*-----( Declare Constants and Pin Numbers )-----*/
// Data wire is plugged into port 10 on the Arduino (can be changed)
#define ONE_WIRE_BUS 10    // NOTE: No ";" on #define  
#define I2C_ADDR    0x20  // Define I2C Address for the PCF8574A 
//---(Following are the PCF8574 pin assignments to LCD connections )----
// This are different than earlier/different I2C LCD displays
#define BACKLIGHT_PIN  7
#define En_pin  4
#define Rw_pin  5
#define Rs_pin  6
#define D4_pin  0
#define D5_pin  1
#define D6_pin  2
#define D7_pin  3

#define  LED_OFF  0
#define  LED_ON  1

/*-----( Declare objects )-----*/  
// Setup a oneWire instance to communicate with any OneWire devices 
// (not just Maxim/Dallas temperature ICs)
OneWire oneWire(ONE_WIRE_BUS);
// Pass address of our oneWire instance to Dallas Temperature. 
DallasTemperature sensors(&oneWire);
LiquidCrystal_I2C  lcd(I2C_ADDR,En_pin,Rw_pin,Rs_pin,D4_pin,D5_pin,D6_pin,D7_pin);

/*-----( Declare Variables )-----*/
float  TempInC  ; //Floating Point variable for C Temperature
float  TempInF  ; //Floating Point variable for F Temperature

void setup()   /*----( SETUP: RUNS ONCE )----*/
{
  sensors.begin();   // Start up the temperature measurement library
  lcd.begin (20,4);  // initialize the lcd 
// Switch on the backlight
  lcd.setBacklightPin(BACKLIGHT_PIN,NEGATIVE);
  lcd.setBacklight(LED_ON);
}// END Setup

void loop()   /*----( LOOP: RUNS OVER AND OVER AGAIN )----*/
{
  sensors.requestTemperatures(); // Send the command to get temperatures
// Reset the display  
  lcd.clear();
  TempInC = sensors.getTempCByIndex(0); // Get and save sensor value
  TempInF = sensors.getTempFByIndex(0); // Get and save sensor value  
  delay(1000);
  lcd.home();
  
// Print our characters on the LCD
  lcd.backlight();  //Backlight ON if under program control
  
  lcd.setCursor(0,0); //Start at character 3 on line 0
  lcd.print("Temp(C) = ");
  lcd.print(TempInC, 2);    // "2" sets the number of decimal place  
  delay(1000);
  
  lcd.setCursor(1,1);
  lcd.print("From YourDuino");
  delay(1000);  
  lcd.setCursor(0,2);
  lcd.print("Temp(F) = ");
  lcd.print(TempInF, 2);    // "2" sets the number of decimal places );
  
  lcd.setCursor(0,3);
  delay(2000);   
  lcd.print("http://YourDuino.com");
  delay(8000);
} // END Loop
