/* YourDuino Example: Servo Joystick Position Freeze
 - Moves two Servomotors (usually on a pan-tilt kit)
   through a range of positions in response to the position
   of a joystick attached to 2 Analog inputs.
   - "Freezes" Servo position with separate "Freeze Button"
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
 - Joystick connected to +5, Gnd, YourDuino Analog inputs 0 and 1
 - Servo connectors plugged on YourDuinoRobo1 port 9 and 10
   - If separate wires:
   - Servo Black to Gnd.
   - Servo Red or Orange (Center wire) to +5V
   - Servo White or Yellow to Signal (Pin 9 or 10)
 - "Freeze" pushbutton connected from pin 6 to ground.
 - V1.01 01/03/14
 Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/
#include <Servo.h>  // Comes with Arduino IDE

/*-----( Declare Constants and Pin Numbers )-----*/
#define ServoHorizontalPIN  3   // Can be changed 3,5,6,9,10,11
#define ServoVerticalPIN    5   // Can be changed 3,5,6,9,10,11
#define FreezeServosPIN     6   // Can be any pin
#define HorizontalPotPin    A0  // Analog input 0 (zero)
#define VerticalPotPin      A1  // Analog input 1 

#define ServoMIN_H  20  // Don't go to very end of servo travel
#define ServoMAX_H  160 // which may not be all the way from 0 to 180. 
#define ServoMIN_V  20  // Don't go to very end of servo travel
#define ServoMAX_V  160 // which may not be all the way from 0 to 180. 

/*-----( Declare objects )-----*/
Servo HorizontalServo;  // create servo object to control a servo
Servo VerticalServo;    // create servo object to control a servo
// a maximum of eight servo objects can be created

/*-----( Declare Variables )-----*/
int HorizontalPotValue;          // User moves the pot.
int HorizontalServoPosition;    // variable to store the servo position

int VerticalPotValue;
int VerticalServoPosition;

boolean FreezePosition = 1; // Freeze the servo position


void setup()   /****** SETUP: RUNS ONCE ******/
{
  HorizontalServo.attach(ServoHorizontalPIN);  // attaches the servo to the servo object
  VerticalServo.attach(ServoVerticalPIN);      // attaches the servo to the servo object
  pinMode(FreezeServosPIN, INPUT_PULLUP);         // Enable pullup
}//--(end setup )---


void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  if ((digitalRead(FreezeServosPIN) == 1)  // Button NOT pushed
      && (FreezePosition == false))

  { // Read Joystick position, follow with servos
    HorizontalPotValue  = analogRead(HorizontalPotPin);  // Get the value as user moves pot
    VerticalPotValue    = analogRead(VerticalPotPin);  // Get the value as user moves pot

    // scale it to use it with the servo (value between MIN and MAX)
    HorizontalServoPosition  = map(HorizontalPotValue, 0, 1023, ServoMIN_H , ServoMAX_H);
    VerticalServoPosition    = map(VerticalPotValue, 0, 1023, ServoMIN_V , ServoMAX_V);

    // tell servos to go to position
    HorizontalServo.write(HorizontalServoPosition);
    VerticalServo.write(VerticalServoPosition);
  }// END: Follow Joystick with servos
  
  
  else // Button WAS pushed
  {
    if ((digitalRead(FreezeServosPIN) == 0)  // Button IS pushed
    && (FreezePosition == false))
    {
      FreezePosition = true;
      delay(1000);
    }
    else
      FreezePosition = false;
      delay(1000);
  }// 

  delay(25);  // wait for the servo to reach the position, debounce button

}//--(end main loop )---

/*-----( Declare User-written Functions )-----*/
//none

//*********( THE END )***********


