/* YourDuino Basic Robot Kit: Test Motors, Motor Driver, Ping
 - WHAT IT DOES: Tests the motor driver, battery connection and
   motors by running them each in both directions. Tests concurrent
   Ultrasonic Sensor Ping.
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
   Pins 9 (Motor1 PWM) and 10 (Motor2 PWM) are predefined, unchangeable
   Motor1A  pin 2   // Motor pins to Driver board
   Motor1B  pin 4
   Motor2A  pin 7
   Motor2B  pin 8
   
 - V1.00 10/28/13
   Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/
#include <YD_MotorDriver1.h>
#include <NewPing.h>

/*-----( Declare Constants  and Pin Numbers )-----*/
// NOTE: Pins 9 (Motor1) and 10 (Motor2) are predefined, unchangeable
#define  Motor1A  2   // Motor pins to Driver board
#define  Motor1B  4
#define  Motor2A  7
#define  Motor2B  8
#define  RampDelay  10
// Ultrasonic Sensor Pins
#define TRIGGER_PIN  5  // Arduino pin tied to trigger pin on the ultrasonic sensor.
#define ECHO_PIN     6  // Arduino pin tied to echo pin on the ultrasonic sensor.
#define MAX_DISTANCE 200 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.


/*-----( Declare objects )-----*/
YD_MotorDriver1 RobotDriver(Motor1A,Motor1B,Motor2A,Motor2B);
NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); // NewPing setup of pins and maximum distance.

/*-----( Declare Variables )-----*/
int i ;  //For looping

unsigned int uS;  // Result of a ping: MicroSeconds
unsigned int cm;  // Distance calculated for ping (0 = outside set distance range)


void setup()   /*----( SETUP: RUNS ONCE )----*/
{
  Serial.begin(9600);
  Serial.println("YourDuino MotorDriver1 Ramp Up / Down Test");
  RobotDriver.init();

}/*--(end setup )---*/


void loop()   /*----( LOOP: RUNS CONSTANTLY )----*/
{
  
/*----( Ramp Motor1 Up and Down both directions )-------*/  
  for (int i = 0; i <= 400; i++)
  {
    RobotDriver.Motor1Speed(i);
    delay(RampDelay);
    PingBlink();    
  }  
  
  for (int i = 400; i >= -400; i--)
  {
    RobotDriver.Motor1Speed(i);
    delay(RampDelay);
    PingBlink();
  } 

  for (int i = -400; i <= 0; i++)
  {
    RobotDriver.Motor1Speed(i);
    delay(RampDelay);
    PingBlink();
  } 


  
/*----( Ramp Motor2 Up and Down both directions )-------*/  
  for (int i = 0; i <= 400; i++)
  {
    RobotDriver.Motor2Speed(i);
    delay(RampDelay);
    PingBlink();    
  }  
  
  for (int i = 400; i >= -400; i--)
  {
    RobotDriver.Motor2Speed(i);
    delay(RampDelay);
    PingBlink();    
  } 

  for (int i = -400; i <= 0; i++)
  {
    RobotDriver.Motor2Speed(i);
    delay(RampDelay);
    PingBlink();    
  } 

  delay(5000);  
}/* --(end main loop )-- */

/*-----( Declare User-written Functions )-----*/
void PingBlink()
{
  uS = sonar.ping(); // Send ping, get ping time in microseconds (uS).
  cm = uS / US_ROUNDTRIP_CM; // Convert ping time to distance in cm 
  if (cm < 15) digitalWrite(13,HIGH); else digitalWrite(13,LOW);
}

/* ( THE END ) */ 
