#include <LED.h>

/* YourDuino Basic Robot Kit: Test Motors, Motor Driver, Ping
 - WHAT IT DOES: Tests the motor driver, battery connection and
   motors by running them each in both directions. Tests concurrent
   Ultrasonic Sensor Ping.
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
   Pins 9 (Motor1 PWM) and 10 (Motor2 PWM) are predefined, unchangeable
   Motor1A  pin 2   // Motor pins to Driver board
   Motor1B  pin 4
   Motor2A  pin 7
   Motor2B  pin 8
   
 - V1.00 10/28/13
   Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/
#include <YD_MotorDriver1.h>
#include <NewPing.h>

/*-----( Declare Constants  and Pin Numbers )-----*/
// NOTE: Pins 9 (Motor1) and 10 (Motor2) are predefined, unchangeable
#define  Motor1A  2   // Motor pins to Driver board
#define  Motor1B  4
#define  Motor2A  7
#define  Motor2B  8
#define  RampDelay  10
#define  StartMoveSpeed  200  // Motor Driver value for start of motion
#define  SlowMoveSpeed   250
#define  MediumMoveSpeed 300
#define  FastMoveSpeed   350

// Ultrasonic Sensor Pins
#define TRIGGER_PIN  5  // Arduino pin tied to trigger pin on the ultrasonic sensor.
#define ECHO_PIN     6  // Arduino pin tied to echo pin on the ultrasonic sensor.
#define MAX_DISTANCE 200 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.
// Servo
#define SERVO_PIN    11  // The "Look Around" servo

/*-----( Declare objects )-----*/
YD_MotorDriver1 RobotDriver(Motor1A,Motor1B,Motor2A,Motor2B);
NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); // NewPing setup of pins and maximum distance.

/*-----( Declare Variables )-----*/
//int i ;  //For looping

unsigned int uS;  // Result of a ping: MicroSeconds
unsigned int cm;  // Distance calculated for ping (0 = outside set distance range)


void setup()   /*----( SETUP: RUNS ONCE )----*/
{
  Serial.begin(9600);
  delay(1000);
  Serial.println("YourDuino Robot Kit Test");
  pinMode(SERVO_PIN,OUTPUT);  
  RobotDriver.init();

}/*--(end setup )---*/


void loop()   /*----( LOOP: RUNS CONSTANTLY )----*/
{
  
/*----( Move Forward)-------*/  
/*
  ForwardMedium(); 
  delay(3000);
  Stop();
  delay(500);
  BackwardFast();
  delay(1000);
  SpinLeft();
  delay(2000);
  Stop();
  delay(2000);
  ForwardMedium(); 
  delay(2000);  
  ForwardSlow(); 
  delay(2000);    
  Stop();
  */
  
  ForwardMedium();   
  PointServo(1500);
  delay(100);  
  PingBlink();
  delay(1000);
  Stop();
  
  ForwardSlow();    
  PointServo(2000); 
  delay(100); 
  PingBlink();   
  delay(1000);  
  Stop();
  
}/* --(end main loop )-- */

/*-----( Declare User-written Functions )-----*/


void ForwardSlow()
{
 RobotDriver.Motor1Speed(SlowMoveSpeed);
 RobotDriver.Motor2Speed(SlowMoveSpeed); 
}

void ForwardMedium()
{
 RobotDriver.Motor1Speed(MediumMoveSpeed);
 RobotDriver.Motor2Speed(MediumMoveSpeed); 
}

void ForwardFast()
{
 RobotDriver.Motor1Speed(FastMoveSpeed);
 RobotDriver.Motor2Speed(FastMoveSpeed); 
}

void BackwardSlow(int HowMuch)
{
 RobotDriver.Motor1Speed(- SlowMoveSpeed);
 RobotDriver.Motor2Speed(- SlowMoveSpeed); 
 delay(HowMuch);
 Stop();  
}

void BackwardMedium(int HowMuch)
{
 RobotDriver.Motor1Speed(- MediumMoveSpeed);
 RobotDriver.Motor2Speed(- MediumMoveSpeed); 
 delay(HowMuch);
 Stop(); 
}

void BackwardFast(int HowMuch)
{
 RobotDriver.Motor1Speed(- FastMoveSpeed);
 RobotDriver.Motor2Speed(- FastMoveSpeed); 
 delay(HowMuch);
 Stop(); 
}

void Stop()
{
 RobotDriver.Motor1Speed(0);
 RobotDriver.Motor2Speed(0); 
}

void SpinLeft(int HowMuch)
{
 RobotDriver.Motor1Speed(MediumMoveSpeed);
 RobotDriver.Motor2Speed(- MediumMoveSpeed);
 delay(HowMuch);
 Stop();
}

void SpinRight(int HowMuch)
{
 RobotDriver.Motor1Speed(MediumMoveSpeed);
 RobotDriver.Motor2Speed(- MediumMoveSpeed);
 delay(HowMuch);
 Stop(); 
}



unsigned int PingBlink()
{
  uS = sonar.ping(); // Send ping, get ping time in microseconds (uS).
  cm = uS / US_ROUNDTRIP_CM; // Convert ping time to distance in cm 
  if (cm < 20) digitalWrite(13,HIGH); else digitalWrite(13,LOW);
  if (cm < 40)
  {
    Serial.print("Target ");
    Serial.print(cm/10, DEC);
    Serial.println(" DangerUnits Away!");
    return(cm%10);
  }
  else
  {
    return(0);
  }
}// end PingBlink


void PointServo(int ServoAngle){//void rotate start

  for (int i=0;i<5;i++)
  {
    digitalWrite(SERVO_PIN,HIGH);
    delayMicroseconds(ServoAngle);
    digitalWrite(SERVO_PIN,LOW);
    delay(20);
  }
}// rotate end


/* ( THE END ) */ 
