/* YourDuinoStarter Example: Web Cloud Data Display for Arduino 
   COSM_ARDUINO_DHT11_TEST1
 - Create Web Server to send data to cosm.com for display/archive 
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
   - 
   - 
 - V1.01 01/10/12
   Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/
#include <dht11.h>      // From Rob Tillaart: http://playground.arduino.cc/Main/DHTLib
#include <SPI.h>        // Standard Arduino Library
#include <Ethernet.h>   // Standard Arduino Library
#include <HttpClient.h> // HttpClient lib : https://github.com/amcewen/HttpClient
#include <Cosm.h>       // Cosm Arduino lib : https://github.com/mnin/CosmArduino


// NOTE: The 2 lines below are from the automatically generated Arduino code that Cosm makes
// when you create a new feed. Cut and paste just these lines for reference

#define API_KEY "sHpN5ykoehI5G3ir2dnBlEdJPXeSAKxDWEVmenduMUNIZz0g" // your Cosm API key
#define FEED_ID 97078 // your Cosm feed ID

/*-----( Declare Constants and Pin Numbers )-----*/
#define        DHT11PIN   2  // The Temperature/Humidity sensor
char           cosmKey[] = API_KEY;
unsigned long  FeedID    = FEED_ID;

/*-----( Declare Variables )-----*/
// MAC address for your Ethernet shield:Change if you have more than one on your network
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };
IPAddress ip(192,168,1,122);   // Available Static IP address on your Network. See Instructions
// Define the strings for our datastream IDs
char sensorId[]  = "Temp C";
char sensorId2[] = "Temp F";
char sensorId3[] = "Humid";
char sensorId4[] = "DewPt";

// Define variables for Sensor data to be sent to Cosm
float TempC, TempF, Humidity, DewPoint;     

int cosmReturn = 0;    // Result Return code from data send


/*-----( Declare objects )-----*/
// NOTE:  DATASTREAM_INT = integer    DATASTREAM_FLOAT = float
// Define the 4 streams
CosmDatastream datastreams[] = {
  CosmDatastream(sensorId, strlen(sensorId),  DATASTREAM_FLOAT),    // define this ID in cosm
  CosmDatastream(sensorId2, strlen(sensorId2), DATASTREAM_FLOAT),   // define this ID in cosm
  CosmDatastream(sensorId3, strlen(sensorId3), DATASTREAM_FLOAT), // define this ID in cosm
  CosmDatastream(sensorId4, strlen(sensorId4), DATASTREAM_FLOAT), // define this ID in cosm
};

// Define the Feed, consisting of the 4 DataStreams
CosmFeed feed(FeedID, datastreams, 4 );  

// Define the Ethernet Library structures
EthernetClient client;
CosmClient cosmclient(client);

dht11 DHT11;  // Declare the DHT11 sensor object
/*------------------( END OF DECLARATION AREA )-----------------------------*/


void setup()   /****** SETUP: RUNS ONCE ******/
{      
   Serial.begin(9600);  //Ready to send debug info to Serial Monitor
  
   Serial.println("Starting datastream upload to Cosm...");
   Serial.println("Starting Ethernet Client...");
   
   Ethernet.begin(mac, ip);
   Serial.println("Ethernet client & IP address OK");  
   
   delay(1000);  //Wait for Server to start up
}//--(end setup )---


 
void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  
/*-----------( Read the DHT11 Temperature / Humidity Sensor, and check for errors. )-----*/  
  int chk = DHT11.read(DHT11PIN);
  Serial.println("=================================");  
  Serial.print("Read sensor: ");
  switch (chk)
  {
    case 0: Serial.println("OK"); break;
    case -1: Serial.println("Checksum error"); break;
    case -2: Serial.println("Time out error"); break;
    default: Serial.println("Unknown error"); break;
  }
  TempC     =  (float)DHT11.temperature,2;
  TempF     =  Fahrenheit(DHT11.temperature);
  Humidity  =  (float)DHT11.humidity;
  DewPoint  =  dewPointFast(DHT11.temperature, DHT11.humidity);
  
/*-----------( Show the results on the Serial Monitor)-----*/   
  Serial.print("Temperature (oC): ");
  Serial.println(TempC);
  Serial.print("Temperature (oF): ");
  Serial.println(TempF);
  Serial.print("Humidity (%): ");
  Serial.println(Humidity);
  Serial.print("Dew PointFast (oC): ");
  Serial.println(DewPoint);

   
 /*-----------( Put sensor values into the 4 datastreams )-----*/    
  // NOTE: Use   setInt() for integer   setFloat() for floating point
  datastreams[0].setFloat(TempC);    // Set sensor values into datastreams 1
  datastreams[1].setFloat(TempF);    // Set sensor values into datastreams 2
  datastreams[2].setFloat(Humidity);  // Set sensor values into datastreams 3
  datastreams[3].setFloat(DewPoint);  // Set sensor values into datastreams 4
  
  
  // NOTE:  getInt for integer, getFloat for floating point
  /*-----------( Show the values in the streams for test/debug )-----*/  
  Serial.println("-----[ Test: Check values inside the Streams ]-----");
  Serial.print("TempC:");
  Serial.println(datastreams[0].getFloat());  // Print datastream to serial monitor
  Serial.print("TempF:");
  Serial.println(datastreams[1].getFloat());  // Print datastream to serial monitor
  Serial.print("Humidity:");
  Serial.println(datastreams[2].getFloat());  // Print datastream to serial monitor
  Serial.print("DewPoint:");
  Serial.println(datastreams[3].getFloat());  // Print datastream to serial monitor
  
  /*-----------( Send the Feed to Cosm.com )-----*/ 
//(while testing/debugging sensor codes, you can comment the following line out.)
   cosmReturn = cosmclient.put(feed,cosmKey);   // Send feed to cosm

   Serial.print("COSM client returned : "); // Get return result code, similar to HTTP code
   if (cosmReturn == 200) { Serial.print(" OK "); }
   Serial.println(cosmReturn);

  
  delay(2000); // Put a delay before the next updates to Cosm
}//--(end main loop )---

/*-----( Declare User-written Functions )-----*/

/*--------( Calculate Fahrenheit from C )----------*/
//Celsius to Fahrenheit conversion
double Fahrenheit(double celsius)
{
        return 1.8 * celsius + 32;
}

/*--------( Calculate Dew Point )----------*/
// reference: http://en.wikipedia.org/wiki/Dew_point
double dewPointFast(double celsius, double humidity)
{
        double a = 17.271;
        double b = 237.7;
        double temp = (a * celsius) / (b + celsius) + log(humidity/100);
        double Td = (b * temp) / (a - temp);
        return Td;
}

//*********( THE END )***********


