
/* CABIN Test 1
 - WHAT IT DOES: Read OPTO22 Inputs, Set OPTO22 Outputs
 - CONNECTIONS:
 - Opto22 DI Mega 38 .. 53
 - Opto22 DO Mega 22 .. 37
 - V1.00 09/11/12
 Questions: terry@yourduino.com */

//***********( ENABLE/DISABLE DEBUG MESSAGES )******************
#define DEBUG
#include <DebugUtils.h>
// Then simply use DEBUG_PRINT("message"); to print your debugging message. 
// Or if you don't want the debug messages to appear simply comment out the DEBUG definition


//****************( Include Needed libraries )************************************
#include <Wire.h>      // I2C Communications Library
#include <RealTimeClockDS1307.h>  //DS1307 Real Time Clock
#include <SoftwareSerial.h>  // For communication to Emic2 Text-To-Speech module
// Module from http://www.grandideastudio.com/portfolio/emic-2-text-to-speech-module/ 
// (Sold by Parallax and Sparkfun.com)

#include <avr/pgmspace.h>    // For Morse Decoder
// This library from:   http://raronoff.wordpress.com/2010/12/16/morse-endecoder/
#include <MorseEnDecoder.h>

// Get 1-wire Library here: http://www.pjrc.com/teensy/td_libs_OneWire.html
#include <OneWire.h>

//Get DallasTemperature Library here:  http://milesburton.com/Main_Page?title=Dallas_Temperature_Control_Library
#include <DallasTemperature.h>


//************************( Declare Constants and Pin Numbers )*************************
#include "iodoc.h"  // This is the TAB "iodoc.h"

//***********************( Declare Variables )*************************************
#include "Variables.h"  // This is the TAB "Variables.h"

//**************************( Declare objects )**********************************
morseDecoder morseInput(morseInPin, MORSE_KEYER, MORSE_ACTIVE_LOW);
SoftwareSerial emicSerial =  SoftwareSerial(rxPin, txPin);




void setup()   /****** SETUP: RUNS ONCE ******/
{
  DEBUG_PRINT("SETUP");
  pinMode(DO_HeatZoneMain, OUTPUT);
  pinMode(DO_HeatZoneOff, OUTPUT);  

  pinMode(rxPin, INPUT);
  pinMode(txPin, OUTPUT);

  Serial.begin(9600); 
  delay(10);
  Serial.println("OLEYIII Cabin Control 3.01");  
  //-----( SET UP MORSE DECODE )--------
  morseInput.setspeed(5);  // 5 words per minute : slow  
  
  //TESTS
  PCIndex = 0;

  //-----( SET UP Text-To-Speech )-----------
  /*  
   emicSerial.begin(9600);             // set the data rate for the SoftwareSerial port
   emicSerial.print('\n');             // Send a CR in case the system is already up
   while (emicSerial.read() != ':');   // When the Emic 2 has initialized and is ready, it will send a single ':' character, so wait here until we receive it
   delay(10);                          // Short delay
   emicSerial.flush();                 // Flush the receive buffer
   emicSerial.print("S System Started. This is Oh Lee.\n");  // Send a string to convert to speech  
   */

}//--(end setup )---


void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  //-------------------------( Decode Morse )---------------------------------
  // Need to call this once per loop
  morseInput.decode();  

  //-------( GetTime:  Sometimes updated from RealTime Clock )-------------------------   
  GetTime();  
  //-------( CheckSensors: Reads Digital Inputs.  Maybe reads Temperatures? )-----------------------
  CheckSensors();  

  //-------( GetHumanInput: Reads Morse, Checks communications interfaces )-----------------------
  GetHumanInput();  

  //-------( ProcessCommands: Changes MODES, SAYS or Displays requested data )--------------------
  ProcessCommands();  

  //-------( ProcessSchedule: Modifies MODES depending on real time/date )--------------------
  ProcessSchedule(); 

  //-------( ProcessMonitors: Checks physical variables and Digital IN against limits, sets MODES. )----
  ProcessMonitors(); 

  //-------( ExecuteModeChanges: Changes logical or physical MODES. Sets digital I/O points. )------
  ExecuteModeChanges(); 

  delay(2000);
}//--(end main loop )---

/*-----( Declare User-written Functions )-----*/
// NOTE: Functions now on Tabs

//*********( THE END )***********


