// ** VARIABLES.H ** 

//----( Morse Decode Pins )--------------------------------------------------
const byte morseInPin = 8;  // Needs to be in this form
#define rxPin 3    // Serial input (connects to Emic 2 SOUT)
#define txPin 5    // Serial output (connects to Emic 2 SIN)

/*-----( Declare Constants and Pin Numbers )-----*/
//-------( Physical Quantity Constants )---------------------------  }

#define GallonsPerCycleAvg   2.25  //Average Gallons per Waterpump cycle}
#define GasGallonsPerHour    1.00  //Gas Burner firing rate}
#define HeatExchangerTooHot  210.0 //Time to dump some heat}
#define HotWaterTempOK       120.0 //Hot enough: general washing}
#define ClothesWasherTempOK  110.0 //Hot enough: ClothesWasher}
#define ShowerTempOK         150.0 //Hot enough: Shower }
#define DishWasherTempOK     160.0 //Hot enough: Dishwasher}
#define HeatExchMinTemp      110.0 //Minimum before ask for more wood }
#define HeatExchDiffOK         3.5 //Minimum before ask for more wood }
#define ShowerModeHoursMax       3 //Number of hour-check times allowed before off}



/*-----( Declare objects )-----*/
//-----( Declare Variables )----- 328=2K 2560=8k SRAM

//-------( COMMAND MODES:       +  Means value is CheckPointed )---
boolean  AlarmMODE         ; // + Alarm system REQUESTED to be armed }
boolean  AlarmON           ; // + Alarm System HAS been armed }
boolean  AlarmTellMODE     ; // Main Alarm has gone off; not yet disarmed}
boolean  AlarmPreAlrMODE   ; // Alarm system tripped; in prealarm delay time}
boolean  LoopHasOpened     ; // Alarm Loop has been tripped somewhere}
                 
boolean  ClotheswasherMODE ; // + ClothesWasher requested, needs hot water}
boolean  ClotheswasherOn   ; // ClothesWasher is now on}
boolean  DishwasherMODE    ; // + Dishwasher power, hot water requested}
boolean  DishwasherOn      ; // DishWasher Power now on}
boolean  DishwasherONDry   ; // Dishwasher Dry Cycle is active}
boolean  HotWaterMODE      ; // + HotWater for general use requested}
boolean  HotWaterOn        ; // HotWater for general use available}
boolean  ShowerMODE        ; // + Hot Shower water requested}
boolean  ShowerOn          ; // Hot Shower water available}

// ---( MODES That control/supply Heat Energy )---------- 
boolean  GasHeatMODE          ; // +
boolean  GasHeatOn            ;
boolean  GasHeatTellMODE      ;
boolean  WoodHeatMODE         ; // +
boolean  WoodHeatOn           ;
boolean  HeatEnergyNeeded     ; // +
boolean  CircPumpMODE         ; // +
boolean  CircPumpOn           ;
boolean  MainZoneHeatMODE     ;
boolean  MainZoneValveOn      ;
boolean  OfficeZoneHeatMODE   ;
boolean  OfficeZoneValveOn    ;
boolean  HETooHotTellMODE     ;

   //---( MODES That control Communications )--------
boolean  MorseRecent ;      /*Morse in progress or recently used*/
//
   //---( MODES That control audio output )----------
boolean  QuietMODE        ;  // +
boolean  VerboseMODE      ;  // +

boolean  DishWasherWHot   ;
boolean  ShowerWHot       ;
boolean  HotWaterWHot     ;
boolean  ClothesWHot      ;
boolean  StokeUpFire      ;

boolean  OutDoorBelowFreeze ;

 //---( MODES for Water Supply )----------
boolean  WaterPumpMODE    ; // +
boolean  WaterPumpON      ;
boolean  WaterPumpTellMODE;

boolean  WellLevelLowSense;
boolean  WellPressTellMODE;

boolean  PlantWaterMODE   ;
boolean  PlantWaterON     ;
//
//*--------( QUANTITIES:  {+}  Means value is CheckPointed )---

long   WaterPumpCycles    ;  // + #times the water pump has cycled ON/OFF }
float  GallonsPerCycle    ;  // # Gallons water pumped per cycle        }
float  WaterPumpGallons   ;  // + # Gallons water pump has pumped         }
                          
float  LoadsOfWood        ;  // + Loads of wood brought in (Cumulative) }
float  FuelGasGallons     ;  // + Gallons of Fuel Gas Consumed (Cumulative) }
float  FuelGasGallonsToday;  // Gallons of Fuel Gas Consumed today}
float  FuelGasGallonsCycle;  // Gallons of Fuel Gas this burn cycle }
float  KiloWattHoursToday ;  // + Electrical energy Consumed today        }
float  KiloWattHours      ;  // + Electrical energy Consumed (Cumulative) }
                          
float  OutDoorTemp        ;  // Outdoor Temperature F.                  }
float  OutDoorTempC       ;  // Outdoor Temperature C.                  }
float  UpstairsTemp       ;  // Indoors Temperature F.                  }
float  OfficeTemp         ;  // Office  Temperature F.                  }
float  HeatExchInputTemp  ;  // Wood Heat Exchanger  Input Temp. F.      }
float  HeatExchOutputTemp ;  // Wood Heat Exchanger Output Temp. F.      }
float  HeatExchInOutDiff  ;  // Wood Heat Exchanger Differential Temp. F.}
float  DailyHighTemp      ;  // + Highest Outdoor Temperature 00:00 to 23:59}
float  OverNightLowTemp   ;  // + Lowest  Outdoor Temperature 00:00 to 23:59}
float  DegreeDaysToday    ;  // + ((Degrees < 70F) * Days)   00:00 to 23:59}
float  DegreeDays         ;  // + ((Degrees < 70F) * Days) (Cumulative    }
word   ShowerModeHourCount;  // Countdown of hours ShowerMode may be on}
word   CallsReceived      ;  // + Phone calls received}


//--------( TIMERS and TIME Variables )-----------------------*)

unsigned long GasBurnerElapsedTimeMs   ; //  {For each individual Run}
unsigned long PhoneElapsedTimeMs       ; //  {For timeouts}
unsigned long PlantWaterElapsedTimeMs  ; //
unsigned long WaterPumpElapsedTimeMs   ; //
unsigned long DishWasherElapsedTimeMs  ; //
unsigned long ShowerElapsedTimeMs      ; //
unsigned long MainHeatZoneElapsedTimeMs; //  


word Hour,         Minute,     Second ,    Sec100 ;
word LastHour, LastMinute, LastSecond ,LastSec100 ;
word Year    , Month     , Day        ,DayOfWeek  ;

//------------( Index Variables for Round-Robin sequencing )-----------------}
byte PCIndex        ;       //ProcessCommands Index         }
byte PSIndex        ;       //ProcessSchedule Index         }
byte PMIndex        ;       //ProcessMonitors Index         }
byte EMCIndex       ;       //ExecuteModeChanges Index      }
byte HETIndex       ;       //Heat Exchanger Temperature Index}
byte HAudIndex      ;       //House Audio Background Index}
byte HAudSelect     ;       //House Audio Recording Index}
byte MRIndex        ;       //Morse Recently used test index}
//------------( MENU Interface Control Variables )---------------------------}
unsigned long   ScanCount  ;  //Number of times this module has been called}


