/* YourDuino Example: 
 - WHAT IT DOES: Temperature, Humidity sent to COSM Web system for graphing, archive
 - SEE the comments after "//" on each line below
 - CONNECTIONS: 
   - Ethernet shield attached to pins 10, 11, 12, 13
   - DHT11 sensor connected to pin 2
 - V1.00 01/08/2013

 Uses version 2.0 of the COSM.com API. 
 Uses the String library, which is part of the Arduino core 

 Based on code by Tom Igoe with input from Usman Haque and Joe Saavedra
 
 This code is in the public domain.
 Questions: terry@yourduino.com
 */
/*-----( Import needed libraries )-----*/
#include <SPI.h>       //Needed for Ethernet Shield
#include <Ethernet.h>  // Ethernet Library
#include <dht11.h>     // DHT11 Sensor Library
#include <Wire.h>

/*-----( Declare Constants and Pin Numbers )-----*/
#define DHT11PIN 2  // The Temperature/Humidity sensor
#define API_KEY "6Bd8YNrVaZir2Y4-JWEsG4GG9bmSAKxJc2JVQTZOekRzYz0g" // your Cosm API key
#define FEED_ID 96820 // your Cosm feed ID
#define USERAGENT      "TerryKingTest2Strings" // user agent is the project name

/*-----( Declare Variables )-----*/
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED}; //Assign MAC address

IPAddress ip(192,168,1,121);    // fill in an available IP address on your network

unsigned long lastConnectionTime = 0;       // last time you connected to the server, in milliseconds
boolean       lastConnected = false;        // state of the connection last time through the main loop
const unsigned long postingInterval = 10*1000;  //delay between updates to pachube.com (ms)

String dataString = "";
String workString = "";

/*-----( Declare objects )-----*/
dht11 DHT11;              // The Sensor Object
EthernetClient client;    // The Ethernet Server

IPAddress server(216,52,233,121);      // numeric IP for api.pachube.com


void setup()   /****** SETUP: RUNS ONCE ******/
{
  Serial.begin(9600); // Open serial communications
  delay(1000);    // give the ethernet module time to boot up:
  Serial.println("TempHumidity Server Starting");
  
  // start the Ethernet connection:
  if (Ethernet.begin(mac) == 0) {
    Serial.println("Failed to configure Ethernet using DHCP");
    // DHCP failed, so use a fixed IP address:
    Ethernet.begin(mac, ip);
  }
}//--(end setup )---


void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  



/*  
  // read the analog sensor:
  int sensorReading = analogRead(A0);   
  // convert the data to a String to send it:

  String dataString = "sensor1,";
  dataString += sensorReading;

  // you can append multiple readings to this String if your
  // pachube feed is set up to handle multiple values:
  int otherSensorReading = analogRead(A1);
  dataString += "\nsensor2,";
  dataString += otherSensorReading;
*/
  // if there's incoming data from the net connection.
  // send it out the serial port.  This is for debugging
  // purposes only:
  if (client.available()) {
    char c = client.read();
    Serial.print(c);
  }

  // if there's no net connection, but there was one last time
  // through the loop, then stop the client:
  if (!client.connected() && lastConnected) {
    Serial.println();
    Serial.println("disconnecting.");
    client.stop();
  }

  // if you're not connected, and ten seconds have passed since
  // your last connection, then connect again and send data: 
  if(!client.connected() && (millis() - lastConnectionTime > postingInterval)) 
  {
    readSensorsBuildDatastring();
    sendData(dataString);
  }
  
  // store the state of the connection for next time through
  // the loop:
  lastConnected = client.connected();
} //--(end main loop )---

/*-----( Declare User-written Functions )-----*/
// this method makes a HTTP connection to the server and sends the data:
void sendData(String thisData) {
  // if there's a successful connection:
  if (client.connect(server, 80)) {
    Serial.println("connecting...");
    // send the HTTP PUT request:
    client.print("PUT /v2/feeds/");
    client.print(FEED_ID);
    client.println(".csv HTTP/1.1");
    client.println("Host: api.pachube.com");
    client.print("X-pachubeApiKey: ");
    client.println(API_KEY);
    client.print("User-Agent: ");
    client.println(USERAGENT);
    client.print("Content-Length: ");
    client.println(thisData.length());

    // last pieces of the HTTP PUT request:
    client.println("Content-Type: text/csv");
    client.println("Connection: close");
    client.println();

    // here's the actual content of the PUT request:
    client.println(thisData);
  } 
  else {
    // if you couldn't make a connection:
    Serial.println("connection failed");
    Serial.println();
    Serial.println("disconnecting.");
    client.stop();
  }
  // note the time that the connection was made or attempted:
  lastConnectionTime = millis();
}

/*-----(ReadSensors-Build Datastring )---------------------------------------------------------*/
//

          /*----(Get sensor reading, calculate and print results )-----------------*/
void readSensorsBuildDatastring()
{
          int chk = DHT11.read(DHT11PIN);

          Serial.print("Read sensor: ");
          
          switch (chk)
          {
          case 0: 
            Serial.println("DHT11 READ OK"); 
            break;
          case -1: 
            Serial.println("DHT11 Checksum error"); 
            break;
          case -2: 
            Serial.println("DHT11 Time out error"); 
            break;
          default: 
            Serial.println("DHT11 Unknown error"); 
            break;
          }  
          
         

          Serial.print("Temperature (C): ");
          Serial.println((float)DHT11.temperature, 1);  
          Serial.println("<br />");  
          
          Serial.print("TryOtherFormat (C): ");
          Serial.println(DHT11.temperature);  
          Serial.println("<br />");          
         
          Serial.print("Temperature (F): ");
          Serial.println(Fahrenheit(DHT11.temperature), 1);
          Serial.println("<br />");  
          
          Serial.print("Humidity (%): ");
          Serial.println((float)DHT11.humidity, 0);  
          Serial.println("<br />");   

/*          Serial.print("Temperature (K): ");
          Serial.println(Kelvin(DHT11.temperature), 1);
          Serial.println("<br />");   

          Serial.print("Dew Point (C): ");
          Serial.println(dewPoint(DHT11.temperature, DHT11.humidity));
          Serial.println("<br />");   

          Serial.print("Dew PointFast (C): ");
          Serial.println(dewPointFast(DHT11.temperature, DHT11.humidity));
          Serial.println("<br />");   
*/

  workString = String(DHT11.temperature);          
  dataString = "TempC,";
  dataString += workString;
  
  workString = String(DHT11.humidity);  
  dataString += "\nHumidity,";
  dataString += workString;  
  
  
          Serial.print("workString");
          Serial.println(workString);
          Serial.print("dataString");
          Serial.println(dataString);    
          


//  dataString += "\nsensor2,";
//  dataString += otherSensorReading; 


}
          /*--------( End Sensor Read )--------------------------------*/

/*---------------------------------------------------------------------------*/
//
//Celsius to Fahrenheit conversion
double Fahrenheit(double celsius)
{
  return 1.8 * celsius + 32;
}
/*---------------------------------------------------------------------------*/
//Celsius to Kelvin conversion
double Kelvin(double celsius)
{
  return celsius + 273.15;
}
/*---------------------------------------------------------------------------*/
// dewPoint function NOAA
// reference: http://wahiduddin.net/calc/density_algorithms.htm 
double dewPoint(double celsius, double humidity)
{
  double A0= 373.15/(273.15 + celsius);
  double SUM = -7.90298 * (A0-1);
  SUM += 5.02808 * log10(A0);
  SUM += -1.3816e-7 * (pow(10, (11.344*(1-1/A0)))-1) ;
  SUM += 8.1328e-3 * (pow(10,(-3.49149*(A0-1)))-1) ;
  SUM += log10(1013.246);
  double VP = pow(10, SUM-3) * humidity;
  double T = log(VP/0.61078);   // temp var
  return (241.88 * T) / (17.558-T);
}
/*---------------------------------------------------------------------------*/
// delta max = 0.6544 wrt dewPoint()
// 5x faster than dewPoint()
// reference: http://en.wikipedia.org/wiki/Dew_point
double dewPointFast(double celsius, double humidity)
{
  double a = 17.271;
  double b = 237.7;
  double temp = (a * celsius) / (b + celsius) + log(humidity/100);
  double Td = (b * temp) / (a - temp);
  return Td;
}

//*********( THE END )***********
