/* YourDuinoStarter Example: Sketch Template
 - WAVEHR_IR_Control
 - WHAT IT DOES: Plays wave files on Wave shield, Controlled by IR REmote
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
   - IR Reciever on Pin 6
   - Wave Shield
 - V1.00 04/11/13
   Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/
// IR Remote Library: https://github.com/shirriff/Arduino-IRremote
#include <IRremote.h>
// WaveHC Library: https://code.google.com/p/wavehc/
#include "WaveUtil.h"
#include "WaveHC.h" 

/*-----( Declare Constants and Pin Numbers )-----*/
int RECV_PIN = 6;

/*-----( Declare objects )-----*/
SdReader card;    // This object holds the information for the card
FatVolume vol;    // This holds the information for the partition on the card
FatReader root;   // This holds the information for the filesystem on the card
FatReader f;      // This holds the information for the file we'll play
WaveHC wave;      // This is the only wave (audio) object, since we will only play one at a time
IRrecv irrecv(RECV_PIN);
decode_results results;

/*-----( Declare Variables )-----*/


void setup()   /****** SETUP: RUNS ONCE ******/
{
  Serial.begin(9600);
  
  putstring("Free RAM: ");       // This can help with debugging, running out of RAM is bad
  Serial.println(freeRam());      // if this is under 150 bytes it may spell trouble!
  
  //  if (!card.init(true)) { //play with 4 MHz spi if 8MHz isn't working for you
  if (!card.init()) {         //play with 8 MHz spi (default faster!)  
    putstring_nl("Card init. failed!");  // Something went wrong, lets print out why
    sdErrorCheck();
    while(1);                            // then 'halt' - do nothing!
  }
  

// enable optimize read - some cards may timeout. Disable if you're having problems
  card.partialBlockRead(true);
 
// Now we will look for a FAT partition!
  uint8_t part;
  for (part = 0; part < 5; part++) {     // we have up to 5 slots to look in
    if (vol.init(card, part)) 
      break;                             // we found one, lets bail
  }
  if (part == 5) {                       // if we ended up not finding one  :(
    putstring_nl("No valid FAT partition!");
    sdErrorCheck();      // Something went wrong, lets print out why
    while(1);                            // then 'halt' - do nothing!
  }
  
  // Lets tell the user about what we found
  putstring("Using partition ");
  Serial.print(part, DEC);
  putstring(", type is FAT");
  Serial.println(vol.fatType(),DEC);     // FAT16 or FAT32?
  
  // Try to open the root directory
  if (!root.openRoot(vol)) {
    putstring_nl("Can't open root dir!"); // Something went wrong,
    while(1);                             // then 'halt' - do nothing!
  }


  irrecv.enableIRIn(); // Start the receiver

  // Whew! We got past the tough parts.
  putstring_nl("Ready!");

}//--(end setup )---



void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
    //putstring_nl("Playing DO.WAV");
     // playcomplete("DO.WAV"); 


    if (irrecv.decode(&results)) {
     //Serial.println(results.value, HEX);
     //Serial.println(results.value);

     switch(results.value) {
       case 0x4B3408F7: Serial.println("1");  playfile("CUE01.WAV"); break;
       case 0x4B348877: Serial.println("2");  playfile("CUE02.WAV"); break;
       case 0x4B3448B7: Serial.println("3");  playfile("CUE09.WAV"); break;
       case 0x4B34C837: Serial.println("4");  playfile("CUE04.WAV"); break;
       case 0x4B3418E7: Serial.println("5");  playfile("CUE05.WAV"); break;
       case 0x4B349867: Serial.println("6");  playfile("CUE06.WAV"); break;
       case 0x4B3458A7: Serial.println("7");  playfile("CUE07.WAV"); break;
       case 0x4B3430CF: Serial.println("8");  playfile("CUE08.WAV"); break;
       case 0x4B34B04F: Serial.println("9");  playfile("CUE10.WAV"); break;
     }
        
     irrecv.resume(); // Receive the next value
  }

}//--(end main loop )---


/*-----( Declare User-written Functions )-----*/


//----------------( freeRam )------------------------------------------------------------------
// this handy function will return the number of bytes currently free in RAM, great for debugging!   
int freeRam(void)
{
  extern int  __bss_end; 
  extern int  *__brkval; 
  int free_memory; 
  if((int)__brkval == 0) {
    free_memory = ((int)&free_memory) - ((int)&__bss_end); 
  }
  else {
    free_memory = ((int)&free_memory) - ((int)__brkval); 
  }
  return free_memory; 
} 

//----------------( sdErrorCheck )------------------------------------------------------------------

void sdErrorCheck(void)
{
  if (!card.errorCode()) return;
  putstring("\n\rSD I/O error: ");
  Serial.print(card.errorCode(), HEX);
  putstring(", ");
  Serial.println(card.errorData(), HEX);
  while(1);
}

//----------------( playcomplete )------------------------------------------------------------------
// Plays a full file from beginning to end with no pause.
void playcomplete(char *name) {
  // call our helper to find and play this name
  playfile(name);
  while (wave.isplaying) {
  // do nothing while its playing
  }
  // now its done playing
}


//----------------( playfile)------------------------------------------------------------------
void playfile(char *name) {
  // see if the wave object is currently doing something
  if (wave.isplaying) {// already playing something, so stop it!
    wave.stop(); // stop it
  }
  // look in the root directory and open the file
  if (!f.open(root, name)) {
    putstring("Couldn't open file ");
    putstring(":");
    Serial.print(name);
    putstring_nl(":");
    return;
  }
  // OK read the file and turn it into a wave object
  if (!wave.create(f)) {
    putstring_nl("Not a valid WAV"); return;
  }
  
  // ok time to play! start playback
  wave.play();
}

//*********( THE END )***********

