
// Author: Clinton Sam
// Disclaimer: Written by an entry-level hobbiest. Feel free to improveupon this code.
// Modified a bit by Terry King 12/2/13

// This sketch spins the 5v stepper motor #28BYJ-48.

#include <Stepper.h>

// CONST DECLARATIONS

const float stepAngle = 5.625; // 5v stepper motor #28BYJ-48
const byte rpmAdjustmentValue = 30; // Setting the RPMs to 30 results in an approximate 1 minute revolution.
const int internalStepsPerRevolution = 360/stepAngle; // full steps
const int fullStepsPerRevolution = internalStepsPerRevolution*32;
// Variables
float rpm;

// OBJECT INSTANCES

Stepper motor(internalStepsPerRevolution, 8, 10, 9, 11); // pin order is set as stated in the documentation for the motor #28BYJ-48.

// ------------- MAIN-------------- //

void setup()
{
  motor.setSpeed(setRPM(15)); // set the speed of the motor to 15 rotations per minute;
}

void loop()
{
  spinMotorOneRotation(motor, 1);
  delay(2000);
  spinMotorOneRotation(motor, 0.25);
  delay(2000);
  spinMotorOneRotation(motor, -0.5);
  delay(2000);
  spinMotorOneRotation(motor, -0.75);
  delay(2000);
}

// ----------- END MAIN

// FUNCTION DEFINITIONS

float setRPM(float a)
{
  if (a < 0) rpm = 0;
  else if (a > 15.6) rpm = 15.6; // 15.6 is the maximum speed for the 5v stepper motor #28BYJ-48
  else rpm = a * rpmAdjustmentValue;
  return rpm;
}

void spinMotorOneRotation(Stepper a_motor, float a_rotations)
{
  a_rotations *= fullStepsPerRevolution; // 1 rotation = 1 full revolution.0.25 rotation = 1/4 revolution;
  a_motor.step(a_rotations);
}
