/* YourDuinoStarter Example: 
   TerryKing_Potentiometer1
 - Testing: sending analog data from Potentiometer
 - Testing: Send text message depending on voltage
 - Test of sending data to ksduino. See: https://ksduino.org/
 
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
 - Potentiometer: +5V, Gnd, Signal to PIN 2

 - 
 * KSduino library & examples is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.  
 
 - V1.00` 15 Jan 2013
 Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/

// #include <SPI.h>          // Standard Arduino Library
#include <SPI.h>          // Standard Arduino Library
#include <EthernetUdp.h>  // ksduino library from https://ksduino.org/?downloads
#include <KSduino.h>      // ksduino library from https://ksduino.org/?downloads

/*-----( Declare Constants and Pin Numbers )-----*/
#define PotentiometerPin A2

/*-----(THESE VALUES MUST BE CHANGED FOR EVERY NEW USER AND DEVICE )-----*/

// device ID & Password (received from "add New Device" at http://ksduino.org/?devices )
unsigned int deviceID   = 3324;
unsigned int devicePwd  = 1234;  // MUST be a 4-digit number  

/*-----(THESE VALUES MUST BE CHANGED ONCE for each new Ethernet Interface on your local network  )-----*/
// Available IP Address on YOUR LOCAL NETWORK (see Instructions)
byte ip[] = { 192, 168, 1, 126 };  

/*-----(UNIQUE MAC ADDRESS for your Ethernet shield/module  )-----*/
// Use this one unless your unit has an assigned MAC address (may be on a sticker on board)
// If you have 2 or more Arduinos on your local network their MAC must be different
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0x0D };

/*-----(DO NOT CHANGE THESE VALUES  )-----*/
unsigned int port       = 58833;              // Local UDP port number
byte         serverIp[] = { 178,63,53,233 };  // KSduino Server address & port
unsigned int serverPort = 9930;               // Server UDP port number

/*-----( Declare objects )-----*/
KSduino ksd (deviceID, devicePwd, serverIp, serverPort); // Create KSduino class

/*-----( Declare Global Variables )-----*/
// Define variables for Sensor data 
int    AnalogValue;  // Value 0..1024 returned from 10-bit analog input
float  Voltage;      // Variable from potentiometer
char   message_buf[32];  // the message buffer 32 bytes length

// -------( End of Declaration section )--------------------------------------------

void setup()   /****** SETUP: RUNS ONCE ******/
{
  Serial.begin(9600);  //Ready to send debug info to Serial Monitor
  Serial.println("Starting Data send to ksduino..."); 
  
  ksd.begin (mac, ip, port);     // Start ksduino 
}//--(end setup )---

// ----------------------------------------------------------------------------------

void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  
  /*-----------( Read the DHT11 Temperature / Humidity Sensor, and check for errors. )-----*/

  Serial.println("=================================");  
  Serial.print("Read Potentiometer value: ");
  

  AnalogValue = analogRead(PotentiometerPin);  
  Serial.print("Analog Raw Value = ");    
  Serial.println(AnalogValue);  
  
  Voltage = map(AnalogValue, 0, 1023, 0, 5000);
  Voltage = Voltage * .001;
  
  Serial.print("Analog Voltage = ");  
  Serial.println(Voltage);   
  
  strcpy(message_buf, "Voltage OK");
  if (Voltage < 1.0) { strcpy(message_buf, "Voltage Too LOW"); }
  if (Voltage > 4.0) { strcpy(message_buf, "Voltage Too HIGH"); }  
   
  // Build Data Packet to Send parameters to KSduino server
  ksd.beginPacket ();
   ksd.addParameter ("f1",  Voltage);  
   ksd.addMessage (message_buf);  
  ksd.endPacket ();


// Send data to ksduino Server as UDP packet
  ksd.update ();  

//-------( Delay before next update loop )--------------------------------------
  delay (250); // Send 4 values per second!
  
}//--(end main loop )---


/*------------------( Declare User-written Functions )-----------------------*/


//*********( THE END )***********


