/* YourDuinoStarter Example: 
 TerryKing_Temp_Humid_01
 - Test of sending data to ksduino. See: https://ksduino.org/
 - Sending Temperature and Humidity data from  DHT11 Sensor
 - Read DHT11 type sensor. See: http://arduino-info.wikispaces.com/DHT11-Humidity-TempSensor 
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
 - DHT11 Sensor: +5V, Gnd, Signal to PIN 2

 - 
 * KSduino library & examples is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.  
 
 - V1.04` 15 Jan 2013
 Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/
#include <dht11.h>        // From Rob Tillaart: http://playground.arduino.cc/Main/DHTLib
#include <SPI.h>          // Standard Arduino Library
#include <EthernetUdp.h>  // ksduino library from https://ksduino.org/?downloads
#include <KSduino.h>      // ksduino library from https://ksduino.org/?downloads

/*-----( Declare Constants and Pin Numbers )-----*/
#define        DHT11PIN   2  // The Temperature/Humidity sensor

/*-----(THESE VALUES MUST BE CHANGED FOR EVERY NEW USER AND DEVICE )-----*/

// device ID & Password (received from "add New Device" at http://ksduino.org/?devices )
unsigned int deviceID   = 3322;
unsigned int devicePwd  = 1234;  // MUST be a 4-digit number  

/*-----(THESE VALUES MUST BE CHANGED ONCE for each new Ethernet Interface on your local network  )-----*/
// Available IP Address on YOUR LOCAL NETWORK (see Instructions)
byte ip[] = { 192, 168, 1, 126 };  

/*-----(UNIQUE MAC ADDRESS for your Ethernet shield/module  )-----*/
// Use this one unless your unit has an assigned MAC address (may be on a sticker on board)
// If you have 2 or more Arduinos on your local network their MAC must be different
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0x0D };

/*-----(DO NOT CHANGE THESE VALUES  )-----*/
unsigned int port       = 58833;              // Local UDP port number
byte         serverIp[] = { 178,63,53,233 };  // KSduino Server address & port
unsigned int serverPort = 9930;               // Server UDP port number

/*-----( Declare objects )-----*/
KSduino ksd (deviceID, devicePwd, serverIp, serverPort); // Create KSduino class
dht11 DHT11;  // Create the DHT11 sensor object

/*-----( Declare Global Variables )-----*/
// Define variables for Sensor data 
float TempC, TempF, Humidity, DewPoint;   // Variables from DHT11 Sensor, calculations.

// -------( End of Declaration section )--------------------------------------------

void setup()   /****** SETUP: RUNS ONCE ******/
{
  Serial.begin(9600);  //Ready to send debug info to Serial Monitor
  Serial.println("Starting Data send to ksduino..."); 
  
  ksd.begin (mac, ip, port);     // Start ksduino 
}//--(end setup )---

// ----------------------------------------------------------------------------------

void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  
  /*-----------( Read the DHT11 Temperature / Humidity Sensor, and check for errors. )-----*/

  Serial.println("=================================");  
  Serial.print("Read sensor: ");

  int chk = DHT11.read(DHT11PIN);  
  switch (chk) 
  {
  case  0:  Serial.println("OK");  break; 
  case -1:  Serial.println("Checksum error");  break;
  case -2:  Serial.println("Time out error");  break;
  default:  Serial.println("Unknown error");   break;
  }// End case  
  
  TempC     =  (float)DHT11.temperature,2;     // Value from Sensor
  TempF     =  Fahrenheit(DHT11.temperature);  // Computed value from C
  Humidity  =  (float)DHT11.humidity;          // Value from Sensor
  DewPoint  =  dewPointFast(DHT11.temperature, DHT11.humidity); // Computed Value  
  
  // Send parameters to KSduino server
  ksd.beginPacket ();
  ksd.addParameter ("f1",  TempC);  
  ksd.addParameter ("f2",  TempF);  
  ksd.addParameter ("f3",  Humidity);  
  ksd.addParameter ("f4",  DewPoint);  
  ksd.endPacket ();


// Send data to ksduino Server as UDP packet
  ksd.update ();  

//-------( Delay before next update loop )--------------------------------------
  delay (2000);
  
}//--(end main loop )---


/*------------------( Declare User-written Functions )-----------------------*/

/*--------( Calculate Fahrenheit from C )----------*/

double Fahrenheit(double celsius)
{
  return 1.8 * celsius + 32;
}

/*------------( Calculate Dew Point )---------------*/
// reference: http://en.wikipedia.org/wiki/Dew_point
double dewPointFast(double celsius, double humidity)
{
  double a = 17.271;
  double b = 237.7;
  double temp = (a * celsius) / (b + celsius) + log(humidity/100);
  double Td = (b * temp) / (a - temp);
  return Td;
}
//*********( THE END )***********


