/* YourDuinoStarter Example: 
 TerryKing_Temperature_DS18B20_1
 - Testing: sending temperature data from DS18B20 electronic thermometers
 - Testing: Send text message depending on temperature
 - Test of sending data to ksduino. See: https://ksduino.org/
 
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
 - DS18B20"  Gnd, +5V, Signal to pin 2
 
 - 
 * KSduino library & examples is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.  
 
 - V1.00` 16 Jan 2013
 Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/

#include <SPI.h>          // Standard Arduino Library: needed for Ethernet
#include <EthernetUdp.h>  // ksduino library from https://ksduino.org/?downloads
#include <KSduino.h>      // ksduino library from https://ksduino.org/?downloads

// Get 1-wire Library here: http://www.pjrc.com/teensy/td_libs_OneWire.html
#include <OneWire.h>      // 1-wire bus including DS18B20 

//Get DallasTemperature Library here:  http://milesburton.com/Main_Page?title=Dallas_Temperature_Control_Library
#include <DallasTemperature.h>  // Read temperature from DS18B20

/*-----( Declare Constants and Pin Numbers )-----*/
#define ONE_WIRE_BUS 2 /*-(Connect to Pin 2 )-*/

/*-----(THESE VALUES MUST BE CHANGED FOR EVERY NEW USER AND DEVICE )-----*/

// device ID & Password (received from "add New Device" at http://ksduino.org/?devices )
unsigned int deviceID   = 3326;
unsigned int devicePwd  = 1234;  // MUST be a 4-digit number  

/*-----(THESE VALUES MUST BE CHANGED ONCE for each new Ethernet Interface on your local network  )-----*/
// Available IP Address on YOUR LOCAL NETWORK (see Instructions)
byte ip[] = { 192, 168, 1, 126 };  

/*-----(UNIQUE MAC ADDRESS for your Ethernet shield/module  )-----*/
// Use this one unless your unit has an assigned MAC address (may be on a sticker on board)
// If you have 2 or more Arduinos on your local network their MAC must be different
byte mac[] = { 
  0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0x0D };

/*-----(DO NOT CHANGE THESE VALUES  )-----*/
unsigned int port       = 58833;              // Local UDP port number

// byte         serverIp[] = {   178,63,53,233 };  // KSduino Server address & port (Original)
byte         serverIp[] = {   174,129,200,235 };  // KSduino Server address & port (Test)
unsigned int serverPort = 9930;               // Server UDP port number

/*-----( Declare objects )-----*/
KSduino ksd (deviceID, devicePwd, serverIp, serverPort); // Create KSduino class
OneWire ourWire(ONE_WIRE_BUS);  // Set up a oneWire instance 

/* Tell Dallas Temperature Library to use oneWire Library */
DallasTemperature sensors(&ourWire);

/*-----( Declare Global Variables )-----*/
// Define variables for Sensor data 
int    Therm1C;  // Value returned from DS18B20 chip
int    Therm1F;  // Value returned from DS18B20 chip

char   message_buf[32];  // the message buffer 32 bytes length

// -------( End of Declaration section )--------------------------------------------

void setup()   /****** SETUP: RUNS ONCE ******/
{
  Serial.begin(9600);  //Ready to send debug info to Serial Monitor
  Serial.println("Starting Data send to ksduino..."); 

  ksd.begin (mac, ip, port);   // Start ksduino 
  sensors.begin();             // Start up the DallasTemperature library
}//--(end setup )---

// ----------------------------------------------------------------------------------

void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{

  /*-----------( Read the DS18B20 Temperature )-----*/

  Serial.println("=================================");  

  Serial.println();
  Serial.print("Requesting temperature...");
  sensors.requestTemperatures(); // Send the command to get temperatures
  Serial.println("DONE");
  
  Therm1C = sensors.getTempCByIndex(0); 
  Serial.print("Device 1 (index 0) = ");  
  Serial.print(Therm1C); 
  Serial.println(" Degrees C");
  
  Therm1F = sensors.getTempFByIndex(0);  
  Serial.print("Device 1 (index 0) = ");
  Serial.print(Therm1F); 
  Serial.println(" Degrees F");  


  strcpy(message_buf, "Indoor Temp");
  if (Therm1F > 75.0) { strcpy(message_buf, "Turn Down Stove!"); }
  if (Therm1F < 65.0) { strcpy(message_buf, "Add Wood to Stove!"); }


  // Build Data Packet to Send parameters to KSduino server
  ksd.beginPacket ();
  ksd.addParameter ("f1",  Therm1C);  
  ksd.addParameter ("f2",  Therm1F);     
  ksd.addMessage (message_buf);  
  ksd.endPacket ();


  // Send data to ksduino Server as UDP packet
  ksd.update ();  

  //-------( Delay before next update loop )--------------------------------------
  delay (5000); // Send every 2 seconds

}//--(end main loop )---


/*------------------( Declare User-written Functions )-----------------------*/


//*********( THE END )***********




