/* YourDuinoStarter Example: 
 TerryKing_Test_ksduino_05
 - Test of sending data to ksduino. See: https://ksduino.org/
 - Based on example simple_03 by kirill@ksduino.org
 - SEE the comments after "//" on each line below
 - CONNECTIONS:
 - None
 - 
 * KSduino library & examples is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.  
 
 - V1.03 14 Jan 2013
 Questions: terry@yourduino.com */

/*-----( Import needed libraries )-----*/
#include <SPI.h>          // Standard Arduino Library
#include <EthernetUdp.h>  // ksduino library from https://ksduino.org/?downloads
#include <KSduino.h>      // ksduino library from https://ksduino.org/?downloads

/*-----( Declare Constants and Pin Numbers )-----*/

/*-----(THESE VALUES MUST BE CHANGED FOR EVERY NEW USER AND DEVICE )-----*/

// device ID & Password (received from "add New Device" at http://ksduino.org/?devices )
unsigned int deviceID   = 3321;
unsigned int devicePwd  = 1234;  // MUST be a 4-digit number  

/*-----(THESE VALUES MUST BE CHANGED ONCE FOR EACH NEW USER  )-----*/
// Available IP Address on YOUR LOCAL NETWORK (see Instructions)
byte ip[] = { 192, 168, 1, 126 };  

/*-----(UNIQUE MAC ADDRESS for your Ethernet shield/module  )-----*/
// Use this one unless your unit has an assigned MAC address.
// If you have 2 or more Arduinos on your local network their MAC must be different
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0x0D };

/*-----(DO NOT CHANGE THESE VALUES  )-----*/
unsigned int port       = 58833;              // Local UDP port number
byte         serverIp[] = { 178,63,53,233 };  // KSduino Server address & port
unsigned int serverPort = 9930;

/*-----( Declare objects )-----*/
KSduino ksd (deviceID, devicePwd, serverIp, serverPort); // Create KSduino class

/*-----( Declare Global Variables )-----*/
// none
// -------( End of Declaration section )--------------------------------------------

void setup()   /****** SETUP: RUNS ONCE ******/
{
  ksd.begin (mac, ip, port);     // Start KSduino 
}//--(end setup )---

// ----------------------------------------------------------------------------------

void loop()   /****** LOOP: RUNS CONSTANTLY ******/
{
  static byte d_value = LOW;  // Variables for example
  String buffer = "";

  // Cause the d1 value to change back and forth between HIGH and LOW each time. 
  if (d_value == LOW) d_value = HIGH; 
  else  d_value = LOW;      

  // Create text buffer with parameter d1 in it
  // 'buffer' is a String that starts out blank
  buffer += "d1=";  // add "d1=LOW" or "d1=HIGH" to the string 
  buffer += d_value;

  // Send String object with parameters & values to KSduino server
  ksd.sendBuffer (buffer);  

  // Do delay
  delay (1000);

  // At the end of Loop, send the data to ksduino Server
  ksd.update ();  

}//--(end main loop )---


/*-----( Declare User-written Functions )-----*/
//(none)
//*********( THE END )***********


