// Use two lines of an LCD display to show large numbers

// The large font spans one characetr with over 2 LCD display lines
// to give a font which is similar to a 7-segment display

#include <LiquidCrystal.h>
#include "BigFont.h"

// LCD display definitions
#define  LCD_ROWS  2
#define  LCD_COLS  16

// LCD pin definitions
#define  LCD_RS    8
#define  LCD_ENA   9
#define  LCD_D4    4
#define  LCD_D5    LCD_D4+1
#define  LCD_D6    LCD_D4+2
#define  LCD_D7    LCD_D4+3

static LiquidCrystal lcd(LCD_RS, LCD_ENA, LCD_D4, LCD_D5, LCD_D6, LCD_D7);

// Waiting time between numbers in milliseconds
#define  WAIT_DELAY  100

void setup()
{
  Serial.begin(57600);
  
  // initialise LCD display
  lcd.begin(LCD_COLS, LCD_ROWS);
  lcd.print(F("LCD Big Chars"));
  
  // Load characters to the LCD
  createSingle();
//  createDouble();

  delay(WAIT_DELAY);
  lcd.clear();  // clear the display and ...
  lcd.noCursor(); // ... display the cursor
}

void createSingle(void)
// Set up the elements for the single font
{
  for (uint8_t i=0; i<NUM_ELEMENTS; i++)
  {
    lcd.createChar(i, (uint8_t *)&fontSingle[i*ELEMENT_SIZE]);
  }
}

void createDouble(void)
// Set up the elements for the double font
{
  for (uint8_t i=0; i<NUM_ELEMENTS; i++)
  {
    lcd.createChar(i, (uint8_t *)&fontDouble[i*ELEMENT_SIZE]);
  }
}

void displayDigit(const uint8_t curPos, const uint8_t n)
// Display the digit across two lines at the cursor position specified
{
  // Define the elemnts for the top and bottom for each digit
  const uint8_t top[10] = {1, 0, 7, 7, 3, 4, 4, 7, 1, 1};
  const uint8_t bot[10] = {3, 0, 4, 2, 0, 6, 3, 0, 5, 2};

  lcd.setCursor(curPos, 0);
  lcd.write((char)top[n]);
  lcd.setCursor(curPos, 1);
  lcd.write((char)bot[n]);
}
 
void testNumber(uint32_t n)
// Write a test number on the display
{
  lcd.noDisplay();    // suspend LCD display until we have finished writing
  // Display the characters
  for (uint8_t curPos = LCD_COLS-1; curPos>0 && n>0; curPos--)
  {
    displayDigit(curPos, n%10);
    n /= 10;
  }
  lcd.display();      // now show it again
}

void loop(void)
{
  static uint32_t  count = 0;
  
  testNumber(count++);
  delay(WAIT_DELAY);
}

